#
# $Header: /home/cvsuser/public/mod_pgsqllog/contrib/perl-httplog/HttpLogEntry.pm,v 1.1 2002/04/02 12:10:29 franco Exp $
# $Revision: 1.1 $
# $Date: 2002/04/02 12:10:29 $
# $Log: HttpLogEntry.pm,v $
# Revision 1.1  2002/04/02 12:10:29  franco
# * mod_pgsqllog.c: Tested PostgreSQL connection recovery, added additional logging for aborted connections - franco
#
# * added contributed import script by Muness - franco
#
# Revision 1.5  2002/03/25 18:50:32  muness
# removing unused variables
#
# Revision 1.4  2002/03/25 18:47:59  muness
# Using HTTP::Date instead of Date::Parse.
#
# Revision 1.3  2002/03/25 17:27:23  muness
# Response was spelled wrong.  (reponse_status).  Renaming field to request_status.
#
# Revision 1.2  2002/03/22 20:35:23  muness
# *** empty log message ***
#
#
#
# Format of log : %h %l %u %t \"%r\" %>s %b \"%{Referer}i\" \"%{User-Agent}i\"
# where:
#
# %...h:          Remote host
# %...l:          Remote logname (from identd, if supplied)
# %...u:          Remote user (from auth; may be bogus if return status (%s) is 401)
# %...t:          Time, in common log format time format (standard english format)
# %...r:          First line of request
# %...s:          Status.  For requests that got internally redirected, this is
#                 the status of the *original* request --- %...>s for the last.
# %...b:          Bytes sent, excluding HTTP headers. In CLF format
#                 i.e. a '-' rather than a 0 when no bytes are sent.
# %...{Referer}i:  The contents of Referer: header line(s) in the request
#                  sent to the server.
# %...{User-Agent}i:  The contents of User-Agent: header line(s) in the request
#                     sent to the server.
# e.g.:
#98afe72a.ipt.aol.com - - [12/Mar/2000:19:00:02 -0500] "GET /images/calendar/calcover.jpg HTTP/1.1" 200 24238 "https://www.lingerie.com/calendar/calendar2000.html" "Mozilla/4.0 (compatible; MSIE 5.01; AOL 5.0; Windows 98)" 
#Available fields are $logEntry
#->client_host(), InetAddr
###->host(), actual string
###->name(), DNS lookup
###->addr(), IP address
#->ident(), string
#->user(), string
#->request_time(), unix time
#->request(), HttpRequest
###->request_method(), string (GET/POST, etc...)
###->uri(), string
###->cgi(), string
###->http_version(), string
###->isImage(), 1 or 0
#->request_status(), status code
#->response_size(), string - bytes sent to browser
#->referrer(), URL
###->proto()
###->host(), InetAddr (see client_host for fields)
###->uri(), string
###->cgi(), string
#->agent(), string
#->cookie(), string
#->hostip(), string


package HttpLogEntry;

use InetAddr;
use HttpRequest;
use URL;

use HTTP::Date;

use vars qw($AUTOLOAD);
use strict;

use Carp;

my $Debugging = 0; # class debugging flag
#my $init_time = 0; # class debugging flag
#my $construct_time = 0; # class debugging flag

my %fields = (
	client_host		=> undef, # Using InetAddr
	ident			=> undef, # Not sure what this is
	user			=> undef, # User name if authenticated page
	request_time		=> undef, # Need to find/define a date/time for this (day/month/year/time/TimeZone) - later
	request			=> undef, # Using HttpRequest 
	request_status		=> undef, # Status - define a class (?)
	response_size		=> undef, # Bytes sent (convert - to 0)
	referrer		=> undef, # Using URL
	agent			=> undef, # Use a string for now - define a class for this later (?)
	cookie			=> undef, # cookie string - have no idea if this is actually being used
	hostip			=> undef, # hostip - have no idea if this is actually being used
);

sub new {
#	my $start_constructor = time();
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $this  = {
		_permitted => \%fields,
		%fields,
	};
	my $line = shift;
	bless ($this, $class);
	if ($Debugging || $this->{"_DEBUG"}) {
		carp localtime() . "Creating $this";
	}
	if ($line) { 
		$this->_initialize($line); 
		if ($Debugging || $this->{"_DEBUG"}) {
			carp localtime() . "Initializing complete";
		}
	}
#	$construct_time += time() - $start_constructor;
	return $this;
}


sub _initialize {
#	my $start_init = time();
	my $this = shift;

	my $line = shift;
	chomp ($line);
	if ($Debugging || $this->{"_DEBUG"}) {
		carp localtime() . "Initializing $this using $line\n";
	}

	$this->{"line"}=($line);

	my ($host, $ident, $remoteuser, $datetime, $method, $request, $httpversion, $status, $size, $referrer, $agent, $hostip, $cookie, $cgi) = ("", "", "", "", "", "", "", "", "", "", "", "", "", "");

	if ($line =~ m!(.*?) (.*?) (.*?) \[(.*?)\] "(.*?)" (.*?) (.*?) "(.*?)" "(.*?)"!o) {
		($host, $ident, $remoteuser, $datetime, $request, $status, $size, $referrer, $agent, $cookie, $hostip) = 
			($1, $2, $3, $4, $5, $6, $7, $8, $9, $10 || "", $11 || "");
	}

	$this->client_host(new InetAddr($host));
	$this->ident($ident);
	$this->user($remoteuser);
	$datetime=time2str(str2time($datetime));
	$this->request_time($datetime);
	$this->request(new HttpRequest($request));
	$this->request_status($status);
	$this->response_size($size);
	$this->referrer(new URL($referrer));
	$this->agent($agent);
	$this->cookie($cookie);
	$this->hostip($hostip);

#	$init_time += time() - $start_init;
	return;

}

sub line {
	my $this = shift;
	my $param = shift;

	if ($param) {
		$this->_initialize($param);
		#print "setting line to $param \n";
		return;
	} else {
		return $this->{"line"};
	}
}

sub display {
	my $this = shift;
	my @keys;
	my $key;
	if (@_ == 0) {
		@keys = sort keys (%$this);
	} else {
		@keys = @_;
	}
	foreach $key (@keys) {
		print "\t$key => $this->{$key}\n";
		if (ref ($this->{$key}) eq "URL" || ref($this->{$key}) eq "HttpRequest") {
			print "$key details:\n";
			$this->{$key}->display();
			print "\n";
		}
		
	}
}

sub AUTOLOAD {
	my $self = shift;
	my $type = ref($self)
		or croak "$self is not an object";

	my $name = $AUTOLOAD;
	$name =~ s/.*://o;   # strip fully-qualified portion

	unless (exists $self->{_permitted}->{$name} ) {
		croak "Can't access `$name' field in class $type";
	}

	if (@_) {
		return $self->{$name} = _clean_log_var(shift);
	} else {
		return $self->{$name};
	}
}

sub _clean_log_var {
	my $input = shift;
	
	for ($input) {
		s/-//go;
	}

	return $input;
}

sub debug {
	my $this = shift;
	confess "usage: thing->debug(level)"    unless @_ == 1;
	my $level = shift;
	if (ref($this))  {
		$this->{"_DEBUG"} = $level;         # just myself
	} else {
		$Debugging        = $level;         # whole class
	}
}

sub DESTROY {
	my $this = shift;
	if ($Debugging || $this->{"_DEBUG"}) {
		carp localtime() . " Destroying $this";
	}
}

#sub display_time_info {
#	print "construct_time = $construct_time\n";
#	print "init_time = $init_time\n";
#}

sub byTime {
        if (ref($a) eq "HttpLogEntry" && ref($b) eq "HttpLogEntry") {
                $a->request_time() <=> $b->request_time();
	} else {
		$a <=> $b;
	}
}
