
package HttpRequest;

use vars qw($AUTOLOAD);
use strict;

use Carp;

my $Debugging = 0; # class debugging flag

my %fields = (
	request_method		=> undef, # 
	uri			=> undef, # 
	cgi			=> undef, # 
	http_version		=> undef, # 
);

sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $this  = {
		_permitted => \%fields,
		%fields,
	};
	my $request = shift;
	bless ($this, $class);
	if ($Debugging || $this->{"_DEBUG"}) {
		carp "Creating $this";
	}
	if ($request) { 
		$this->_initialize($request); 
	}
	return $this;
}

sub _initialize {
	my $this = shift;
	my $request = shift;
	chomp ($request);
	if ($Debugging || $this->{"_DEBUG"}) {
		carp "Initializing $this using $request";
	}

	$this->{"request"}=($request);

	my ($method, $uri, $cgi, $httpversion) = ("", "", "", "");

	#check request because its often "-" instead of "METHOD URL HTTPVERSION" (why!?)
	if ($request =~ m!^(.*?)\s+(.*?)\s+HTTP/(.*?)$!o) {
		($method, $uri, $httpversion) = ($1, $2, $3);
		$cgi = "";

		if ($uri =~ m/^(.*)\?(.*)$/o) {
			($uri, $cgi) = ($1, $2);
		}
	} else {
		$this->{"request"}="";
	}
		

	$this->request_method($method);
	$this->uri($uri);
	$this->cgi($cgi);
	$this->http_version($httpversion);

}

sub request {
	my $this = shift;
	my $param = shift;

	if ($param) {
		$this->_initialize($param);
		return;
	} else {
		return $this->{"request"};
	}
}

sub isImage() {
	my $this = shift;
	my $uri = $this->uri();

	#if ($uri =~ m/(JPG|JPEG|PNG|GIF|jpg|jpeg|png|gif)$/o) {
	if ($uri =~ m/(JPG|JPEG|PNG|GIF)$/io) {
		return 1;
	}
	return 0;
}
	

sub display {
	my $this = shift;
	my @keys;
	my $key;
	if (@_ == 0) {
		@keys = sort keys (%$this);
	} else {
		@keys = @_;
	}
	foreach $key (@keys) {
		print "\t$key => $this->{$key}\n";
	}
}

sub AUTOLOAD {
	my $self = shift;
	my $type = ref($self)
		or croak "$self is not an object";

	my $name = $AUTOLOAD;
	$name =~ s/.*://o;   # strip fully-qualified portion

	unless (exists $self->{_permitted}->{$name} ) {
		croak "Can't access `$name' field in class $type";
	}

	if (@_) {
		return $self->{$name} = _clean_log_var(shift);
	} else {
		return $self->{$name};
	}
}

sub _clean_log_var {
	my $input = shift;
	
	for ($input) {
		s/-//go;
	}

	return $input;
}

sub debug {
	my $this = shift;
	confess "usage: thing->debug(level)"    unless @_ == 1;
	my $level = shift;
	if (ref($this))  {
		$this->{"_DEBUG"} = $level;         # just myself
	} else {
		$Debugging        = $level;         # whole class
	}
}

sub DESTROY {
	my $this = shift;
	if ($Debugging || $this->{"_DEBUG"}) {
		carp "Destroying $this";
	}
}
