
package InetAddr;

use Net::hostent;
use Socket;

use vars qw($AUTOLOAD);
use strict;

use Carp;

my $Debugging = 0; # class debugging flag


my %fields = (
	host		=> undef,
);

sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $this  = {
		_permitted => \%fields,
		%fields,
	};
	my $host = shift;
	bless ($this, $class);
	if ($Debugging || $this->{"_DEBUG"}) {
		carp "Creating $this";
	}
	if ($host) { 
		$this->_initialize($host); 
	}
	return $this;
}

sub _initialize {
	my $this = shift;
	my $host = shift;

	chomp ($host);

	#This is WAY too expensive... find a work around
	#$this->host(gethost($host));
	$this->host($host);
}

sub _gethost() {
	my $this = shift;

	my $gethost = $this->{"_gethost"}; 
	if ($gethost) {
		return $gethost;
	} else {
		my $host = $this->host();
		$gethost = gethost($host);	 
		$this->{"_gethost"} = $gethost;
	}

	return $gethost;
}

sub name {
	my $this = shift;
	return ($this->_gethost()->name);
}

sub addr {
	my $this = shift;
	if ($this->_gethost()) {
		return (inet_ntoa($this->_gethost()->addr));
	} else {
		return "";
	}
}


sub aliases {
	my $this = shift;
	my @aliases;
	
	if ($this->{"aliases"}) {
		@aliases = @{$this->{"aliases"}};
		return \@aliases;
	} else {
		my $host = $this->_gethost();
		my $alias;
		$#aliases = -1;
		if ( @{$host->aliases} > 0 ) {
			for $alias ( @{$host->aliases} ) {
				push @aliases, $alias;
			}
		} 
	}

	$this->{"aliases"}=[@aliases];
	return \@aliases;
}

sub addr_list {
	my $this = shift;
	my @addr_list;
	
	if ($this->{"addr_list"}) {
		@addr_list = @{$this->{"addr_list"}};
		return \@addr_list;
	} else {
		my $host = $this->_gethost();
		if ( @{$host->addr_list} > 1 ) {
			my $addr;
			$#addr_list = -1;
			for $addr ( @{$host->addr_list} ) {
				push @addr_list, inet_ntoa($addr);
			}
		} else {
			$#addr_list = -1;
			push @addr_list, inet_ntoa($host->addr);
		}
	}

	$this->{"addr_list"}=[@addr_list];
	return \@addr_list;
}

sub display {
	my $this = shift;
	my @keys;
	my $key;
	if (@_ == 0) {
		@keys = sort keys (%$this);
	} else {
		@keys = @_;
	}
	foreach $key (@keys) {
		print "\t$key => $this->{$key}\n";
	}
}

sub AUTOLOAD {
	my $self = shift;
	my $type = ref($self)
		or croak "$self is not an object";

	my $name = $AUTOLOAD;
	$name =~ s/.*://o;   # strip fully-qualified portion

	unless (exists $self->{_permitted}->{$name} ) {
		croak "Can't access `$name' field in class $type";
	}

	if (@_) {
		return $self->{$name} = shift;
	} else {
		return $self->{$name};
	}
}

sub DESTROY {
	return;
}
