
package URL;

use InetAddr;

use vars qw($AUTOLOAD);
use strict;

use Carp;

my $Debugging = 0; # class debugging flag

my %fields = (
	proto			=> undef, # 
	host			=> undef, # 
	uri			=> undef, # 
	cgi			=> undef, # 
);

sub new {
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $this  = {
		_permitted => \%fields,
		%fields,
	};
	my $url = shift;
	bless ($this, $class);
	if ($Debugging || $this->{"_DEBUG"}) {
		carp "Creating $this";
	}
	if ($url) { 
		$this->_initialize($url); 
	}
	return $this;
}

sub _initialize {
	my $this = shift;
	my $url = shift;
	chomp ($url);
	if ($Debugging || $this->{"_DEBUG"}) {
		carp "Initializing $this using $url";
	}

	$this->{"url"}=($url);

	my ($proto, $host, $uri, $cgi) = ("", "", "", "");

	#check request because its often "-" instead of "METHOD URL HTTPVERSION" (why!?)
	if ($url =~ m!^(.*?)://(.*?)/(.*?)$!o) {
		($proto, $host, $uri) = ($1, $2, "/" . $3);
		$cgi = "";

		if ($uri =~ m/^(.*)\?(.*)$/o) {
			($uri, $cgi) = ($1, $2);
		} elsif ($uri eq "/") { # e.g. altavista.com?q=keyword
			if ($host =~ m/^(.*)\?(.*)$/o) {
				($host, $cgi) = ($1, $2);
			}
		}
	} else {
		$this->{"url"} = "";
	}

	if ($host =~ m/^(.*)\?(.*)$/o) {
		($host) = ($1);
	}

	$this->proto($proto);
	$this->host(new InetAddr($host));
	$this->uri($uri);
	$this->cgi($cgi);

}

sub extract_search_key {
	my $this = shift;

	my $key;
	my $cgi = $this->cgi();

	if (! $cgi) {
		return;
	}

	if ($cgi =~ m/^(.*?)q=(.*?)$/o) {
		$key=$2;
	}

}
sub url {
	my $this = shift;
	my $param = shift;

	if ($param) {
		$this->_initialize($param);
		return;
	} else {
		return $this->{"url"};
	}
}

sub isImage() {
	my $this = shift;
	my $uri = $this->uri();

	if ($uri =~ m/(JPG|JPEG|PNG|GIF)$/io) {
		return 1;
	}
	return 0;
}
	

sub display {
	my $this = shift;
	my @keys;
	my $key;
	if (@_ == 0) {
		@keys = sort keys (%$this);
	} else {
		@keys = @_;
	}
	foreach $key (@keys) {
		print "\t$key => $this->{$key}\n";
	}
}

sub AUTOLOAD {
	my $self = shift;
	my $type = ref($self)
		or croak "$self is not an object";

	my $name = $AUTOLOAD;
	$name =~ s/.*://o;   # strip fully-qualified portion

	unless (exists $self->{_permitted}->{$name} ) {
		croak "Can't access `$name' field in class $type";
	}

	if (@_) {
		return $self->{$name} = shift;
	} else {
		return $self->{$name};
	}
}

sub debug {
	my $this = shift;
	confess "usage: thing->debug(level)"    unless @_ == 1;
	my $level = shift;
	if (ref($this))  {
		$this->{"_DEBUG"} = $level;         # just myself
	} else {
		$Debugging        = $level;         # whole class
	}
}

sub DESTROY {
	my $this = shift;
	if ($Debugging || $this->{"_DEBUG"}) {
		carp "Destroying $this";
	}
}
