/*
//  mod_pgsqllog.h
//
//
//  Author: Franco Gasperino
//  Date: 04/21/01
//
//  Revision: $Revision: 1.6 $
//  Last Modified: $Date: 2002/01/05 02:32:10 $
//  Last Committer: $Author: franco $
//
//  Copyright (C) 2001 Franco Gasperino
//
//  This program is free software; you can redistribute it and/or modify it under 
//  the terms of the GNU General Public License as published by the Free
//  Software Foundation; either version 2, or (at your option) any later
//  version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT
//  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
//  FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
//  more details.
//
//  You should have received a copy of the GNU General Public License along with
//  this program; if not, write to the Free Software Foundation, Inc., 675
//  Mass Ave, Cambridge, MA 02139, USA.
//
*/

#ifndef __MOD_PGSQLLOG__
#define __MOD_PGSQLLOG__

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>

/* apache includes */
#include "httpd.h"
#include "http_config.h"
#include "http_core.h"
#include "http_log.h"
#include "http_main.h"
#include "http_protocol.h"
#include "http_request.h"
#include "ap_config.h"

/* postgresql includes */
#include <libpq-fe.h>

/* defines & typedefs */
#define MODULE_NAME          ("[mod_pgsqllog]")
#define MALLOC_ERROR         ("Failed to allocate dynamic apache memory!")
#define NULL_STRING          ("\0")
#define MAX_DIRECTIVE_LEN    (256)
#define MAX_STATEMENT_LEN    (8192)
#define MAX_TIMESTAMP_LEN    (100)
#define DEFAULT_HANDLE_COUNT (5)
#define HANDLE_WAIT_TIME     (100)

#ifndef HAVE_SEMUN
/* union for IPC semctl() */
union semun {
    int val;
    struct semid_ds *buf;
    ushort *array;
};
#endif

/* PostgreSQL connection 'handle' */
typedef struct {
    PGconn *connection;

    int connected, semid;
} sqlhandle;

typedef struct {
    char *host, *port, *database, *username, *password;

    char *table;

    sqlhandle *handles;
    unsigned short handle_count;

    short retry_count;
} sqlenv;

/* apache callbacks */
static const char *conf_host_action(cmd_parms *, void *, const char *);
static const char *conf_port_action(cmd_parms *, void *, const char *);
static const char *conf_database_action(cmd_parms *, void *, const char *);
static const char *conf_username_action(cmd_parms *, void *, const char *);
static const char *conf_password_action(cmd_parms *, void *, const char *);
static const char *conf_table_action(cmd_parms *, void *, const char *);
static const char *conf_handle_count_action(cmd_parms *, void *,
					    const char *);
static const char *conf_connect_retry_count_action(cmd_parms *, void *,
						   const char *);

/* module cleanup callbacks */
void module_cleanup(void *);
void child_cleanup(void *);

void init_handler(server_rec *, pool *);
void *server_config_handler(pool *, server_rec *);
int log_handler(request_rec *);

/* internal functions */
int sql_connect(sqlenv *, sqlhandle *, int retry);
int sql_disconnect(sqlhandle *);
int sql_log(request_rec *, sqlenv *);
void sql_error(sqlhandle *, PGresult *);

int lock_create(sqlhandle *, unsigned short keyval);
int lock_destroy(sqlhandle *);
int lock_lock(sqlhandle *);
int lock_unlock(sqlhandle *);

const char *epoch_timestamp(time_t);

#endif				/* ifndef __MOD_PGSQLLOG__ */
